﻿using Microsoft.AspNetCore.Mvc.ModelBinding;
using System.Threading.Tasks;

namespace Falcon.SugarApi.ClaimTicket
{
    /// <summary>
    /// 模型绑定
    /// </summary>
    public class ClaimTicketModelBinding:IModelBinder
    {
        /// <summary>
        /// 票据创建器
        /// </summary>
        public ITicketBuilder Builter { get; set; }
        /// <summary>
        /// 配置
        /// </summary>
        public ClaimTicketOptions Options { get; set; }

        /// <summary>
        /// 通过票据创建器创建模型绑定
        /// </summary>
        /// <param name="builter"></param>
        /// <param name="options"></param>
        public ClaimTicketModelBinding(ITicketBuilder builter,ClaimTicketOptions options) {
            Builter = builter;
            this.Options = options;
        }

        Task IModelBinder.BindModelAsync(ModelBindingContext bindingContext) {
            if(bindingContext.ModelType != typeof(UserTicket)) {
                return FailBind(bindingContext);
            }
            var token = bindingContext.ValueProvider.GetValue(this.Options.HttpHeaderKey).ToString();
            if(token.IsNullOrEmpty()) {
                token = bindingContext.HttpContext.Request.Headers[this.Options.HttpHeaderKey].ToString();
            }
            var model = this.Builter.GetUser(token);
            if(model == null) {
                return FailBind(bindingContext);
            }

            bindingContext.Model = model;
            bindingContext.Result = ModelBindingResult.Success(bindingContext.Model);

            return Task.CompletedTask;
        }

        private static Task FailBind(ModelBindingContext bindingContext) {
            bindingContext.Result = ModelBindingResult.Failed();
            return Task.CompletedTask;
        }
    }
}
