﻿using Falcon.SugarApi.Encryption;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;

namespace Falcon.SugarApi.ClaimTicket
{
    /// <summary>
    /// 票据生成器
    /// </summary>
    public class TicketBuilder:ITicketBuilder
    {
        /// <summary>
        /// 构造票据生成器
        /// </summary>
        /// <param name="options">加密配置</param>
        public TicketBuilder(ClaimTicketOptions options) {
            this.Encryption = options.Encryption;
            this.Serialize = options.JsonSerialize;
            this.Options = options;
        }
        /// <summary>
        /// 加密模块
        /// </summary>
        public IEncryption Encryption { get; }
        /// <summary>
        /// 序列化模块
        /// </summary>
        public ISerialize Serialize { get; }
        /// <summary>
        /// 票据生成配置
        /// </summary>
        public ClaimTicketOptions Options { get; }

        /// <inheritdoc/>
        public virtual UserTicket? GetUser(string ticket) {
            var str = this.Encryption.Decrypt(this.Options.EncryptionKey,ticket);
            if(str.IsNullOrEmpty()) {
                return new UserTicket();
            }
            var list = this.Serialize.Deserialize<List<ClaimKeyValue>>(str);
            if(list == null) {
                return new UserTicket();
            }
            return new UserTicket(list.Select(a => new Claim(a.Key,a.Value)));
        }

        /// <inheritdoc/>
        public virtual string? GetTicket(UserTicket userTicket) {
            if(userTicket == null) {
                return string.Empty;
            }
            var claims = userTicket.Claims;
            if(claims == null || claims.Count == 0) {
                return string.Empty;
            }
            var list = claims.Select(a => new ClaimKeyValue { Key = a.Type,Value = a.Value });
            var str = this.Serialize.Serialize(list);
            var sstr = this.Encryption.Encrypt(this.Options.EncryptionKey,str);
            return sstr;
        }
    }
}
